/*
 * Cube.cpp
 *
 *  Created on: Dec 27, 2009
 *      Author: enash
 */

#include "common.h"

#include "square_template.h"
GLushort colors[] = {ColorRED, ColorYELLOW, ColorGREEN, ColorBLUE, ColorBLACK, ColorORANGE};

Cubette *littleCubes[27];
GLushort *atlasData;

Cube::Cube(Surface *displaySurface, float framesPerSecond) {

	cubeIndex = 0;
	vboIdx = 0;

	surface = displaySurface;
	usecPerFrame = 1000000.0f / framesPerSecond;
	fps = framesPerSecond;

	//load and init textures
	initAtlas();
	glEnable(GL_TEXTURE_2D);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, square_template_texture_width, square_template_texture_height*8, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, atlasData);
	myglTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	myglTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	delete[] atlasData;

	//load and init buffer object
	glGenBuffers(1, &vboName);
	glBindBuffer(GL_ARRAY_BUFFER, vboName);
	// buffer will be Verts(x,y,z)-Normals(x,y,z)-UV(u,v) (8 VERTTYPES) 24 entries / cube 27 cubes
	vboData = new VERTTYPE[8 * 24 * 27];
	initLittleCubes(); // this also loads the vboData array
	glBufferData(GL_ARRAY_BUFFER, sizeof(VERTTYPE) * 8 * 24 * 27, vboData, GL_STATIC_DRAW);

}

void Cube::drawOneFrame(FACE face) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	for(int cubeNum=0; cubeNum<27; cubeNum++)
		littleCubes[cubeNum]->Draw(face);
	eglSwapBuffers(surface->eglDisplay, surface->eglSurface);

}

void Cube::Render(FACE face, float seconds) {

	// renders 1 face turning 90 degrees clockwise

	float degreesPerFrame = 90.f / (seconds * fps);

	surface->makeCurrent();

	myglClearColor(f2vt(0.0f), f2vt(0.0f), f2vt(0.5f), f2vt(1.0f));

	// Enable arrays, bound to the vbo
	// set environmental states
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);

	glBindBuffer(GL_ARRAY_BUFFER, vboName);
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, VERTTYPEENUM, 8 * sizeof(VERTTYPE), 0);
	glEnableClientState(GL_NORMAL_ARRAY);
	glNormalPointer(VERTTYPEENUM, 8 * sizeof(VERTTYPE), (GLvoid *)(3 * sizeof(VERTTYPE)));
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glTexCoordPointer(2, VERTTYPEENUM, 8 * sizeof(VERTTYPE), (GLvoid *)(6 * sizeof(VERTTYPE)));

	if(face == NOFACE) {
		drawOneFrame(face);
		return;
	}

	for(int frame=0; frame<fps*seconds; frame++) {
		// draw it
		drawOneFrame(face);
		rotateFace(face, degreesPerFrame);
		usleep(usecPerFrame);
	}

	updateFaceTransforms(face);
}

void Cube::initLittleCubes() {

	// cubes are numbered from 0 to 26
	// 	Cubette(                    transX,     transY,     transZ,    rotateX,   rotateY,     otateZ,  front,  left,  top)

	// front layer
	littleCubes[ 0] = new Cubette(this, f2vt(-1.0f), f2vt(1.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), RED, GREEN, BLUE); // upper left
	littleCubes[ 1] = new Cubette(this, f2vt(0.0f), f2vt(1.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), RED, NOCOLOR, BLUE); // upper middle
	littleCubes[ 2] = new Cubette(this, f2vt(1.0f), f2vt(1.0f), f2vt(1.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, RED, BLUE); // upper right
	littleCubes[ 3] = new Cubette(this, f2vt(-1.0f), f2vt(0.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), RED, GREEN, NOCOLOR); // middle left
	littleCubes[ 4] = new Cubette(this, f2vt(0.0f), f2vt(0.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), RED, NOCOLOR, NOCOLOR); // middle!
	littleCubes[ 5] = new Cubette(this, f2vt(1.0f), f2vt(0.0f), f2vt(1.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, RED, NOCOLOR); // middle right
	littleCubes[ 6] = new Cubette(this, f2vt(-1.0f), f2vt(-1.0f), f2vt(1.0f), f2vt(90.0f), f2vt(0.0f), f2vt(0.0f), BLACK, GREEN, RED); // bottom left
	littleCubes[ 7] = new Cubette(this, f2vt(0.0f), f2vt(-1.0f), f2vt(1.0f), f2vt(90.0f), f2vt(0.0f), f2vt(0.0f), BLACK, NOCOLOR, RED); // bottom middle
	littleCubes[ 8] = new Cubette(this, f2vt(1.0f), f2vt(-1.0f), f2vt(1.0f), f2vt(90.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, BLACK, RED); // bottom right
	// middle layer
	littleCubes[ 9] = new Cubette(this, f2vt(-1.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, GREEN, BLUE); // upper left
	littleCubes[10] = new Cubette(this, f2vt(0.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, NOCOLOR, BLUE); // upper middle
	littleCubes[11] = new Cubette(this, f2vt(1.0f), f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, NOCOLOR, BLUE); // upper right
	littleCubes[12] = new Cubette(this, f2vt(-1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, GREEN, NOCOLOR); // middle left
	littleCubes[13] = new Cubette(this, f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, NOCOLOR, NOCOLOR); // middle!
	littleCubes[14] = new Cubette(this, f2vt(1.0f), f2vt(0.0f), f2vt(0.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, NOCOLOR, NOCOLOR); // middle right
	littleCubes[15] = new Cubette(this, f2vt(-1.0f), f2vt(-1.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), f2vt(0.0f), BLACK, GREEN, NOCOLOR); // bottom left
	littleCubes[16] = new Cubette(this, f2vt(0.0f), f2vt(-1.0f), f2vt(0.0f), f2vt(90.0f), f2vt(0.0f), f2vt(0.0f), BLACK, NOCOLOR, NOCOLOR); // bottom middle
	littleCubes[17] = new Cubette(this, f2vt(1.0f), f2vt(-1.0f), f2vt(0.0f), f2vt(90.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, BLACK, NOCOLOR); // bottom right
	// back layer
	littleCubes[18] = new Cubette(this, f2vt(-1.0f), f2vt(1.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(0.0f), f2vt(0.0f), BLUE, GREEN, ORANGE); // upper left
	littleCubes[19] = new Cubette(this, f2vt(0.0f), f2vt(1.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(0.0f), f2vt(0.0f), BLUE, NOCOLOR, ORANGE); // upper middle
	littleCubes[20] = new Cubette(this, f2vt(1.0f), f2vt(1.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, BLUE, ORANGE); // upper right
	littleCubes[21] = new Cubette(this, f2vt(-1.0f), f2vt(0.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, GREEN, ORANGE); // middle left
	littleCubes[22] = new Cubette(this, f2vt(0.0f), f2vt(0.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(0.0f), f2vt(0.0f), NOCOLOR, NOCOLOR, ORANGE); // middle!
	littleCubes[23] = new Cubette(this, f2vt(1.0f), f2vt(0.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, NOCOLOR, ORANGE); // middle right
	littleCubes[24] = new Cubette(this, f2vt(-1.0f), f2vt(-1.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(-90.0f), f2vt(0.0f), GREEN, BLACK, ORANGE); // bottom left
	littleCubes[25] = new Cubette(this, f2vt(0.0f), f2vt(-1.0f), f2vt(-1.0f), f2vt(-90.0f), f2vt(-90.0f), f2vt(0.0f), NOCOLOR, BLACK, ORANGE); // bottom middle
	littleCubes[26] = new Cubette(this, f2vt(1.0f), f2vt(-1.0f), f2vt(-1.0f), f2vt(180.0f), f2vt(90.0f), f2vt(0.0f), YELLOW, ORANGE, BLACK); // bottom right
}

GLuint Cube::VBOloadOneCube(VERTTYPE *verts, VERTTYPE *normals, VERTTYPE *uvs, GLushort *indexes) {

	int vertIdx = 0;
	int normalIdx = 0;
	int uvIdx = 0;

	for(int i=0; i<24; i++) {

		// verts
		vboData[vboIdx++] = verts[vertIdx++];
		vboData[vboIdx++] = verts[vertIdx++];
		vboData[vboIdx++] = verts[vertIdx++];

		// normals
		vboData[vboIdx++] = normals[normalIdx++];
		vboData[vboIdx++] = normals[normalIdx++];
		vboData[vboIdx++] = normals[normalIdx++];

		// text coords
		vboData[vboIdx++] = uvs[uvIdx++];
		vboData[vboIdx++] = uvs[uvIdx++];
	}

	// index buffer
	GLuint vboIndexName;
	GLushort vboIndexes[36];

	// scale them
	for(uint i=0; i<sizeof(vboIndexes)/sizeof(vboIndexes[0]); i++)
		vboIndexes[i] = cubeIndex * 24 + indexes[i];

	glGenBuffers(1, &vboIndexName);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, vboIndexName);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(vboIndexes), vboIndexes, GL_STATIC_DRAW);

	cubeIndex++;

	return vboIndexName;
}

void Cube::initAtlas() {

	atlasData = new GLushort[square_template_texture_width*square_template_texture_height*8];	// only need 7, but 8th is to keep dimension as power of 2

	for(int c=0; c<6; c++) {

		int offset = c*square_template_texture_width*square_template_texture_height;

		// set the color
		for(uint i=0; i<sizeof(square_template_texture_rgba)/sizeof(square_template_texture_rgba[0]); i+=4) {
			// technically the template should be all 0 (black) or all 255 (white) but this is "just in case" black/white threshold
			int testColor = square_template_texture_rgba[i] + square_template_texture_rgba[i+1] + square_template_texture_rgba[i+2];
			if(testColor > 100)
				atlasData[i/4 + offset] = colors[c];
			else
				atlasData[i/4 + offset] = ColorWHITE;
		}

		// add a border
		for(int i=0; i<square_template_texture_width; i++) {
			atlasData[i + offset] = ColorBLACK;
			atlasData[square_template_texture_width*(square_template_texture_height-1)+i + offset] = ColorBLACK;
		}
		for(int i=0; i<square_template_texture_height; i++) {
			atlasData[i*square_template_texture_width + offset] = ColorBLACK;
			atlasData[(i+1)*square_template_texture_width-1 + offset] = ColorBLACK;
		}
	}

	// 7th is color of inside face
	int offset = 6*square_template_texture_width*square_template_texture_height;
	for(int i=0; i<square_template_texture_width*square_template_texture_height; i++)
		atlasData[i + offset] = ColorCHARCOAL;
}

Cube::~Cube() {
	// TODO Auto-generated destructor stub
}
