/*
 * Cubette.cpp
 *
 *  Created on: Dec 23, 2009
 *      Author: enash
 */

#include "common.h"
#include "CubeVerts.h"

Cubette::Cubette(void *cube, VERTTYPE transX, VERTTYPE transY, VERTTYPE transZ,
		VERTTYPE rotateX, VERTTYPE rotateY, VERTTYPE rotateZ, CUBECOLOR front, CUBECOLOR left, CUBECOLOR top) {

	VERTTYPE myuvs[48];
	setTextureCoords(front, left, top, myuvs);
	indexVBO = ((Cube *)cube)->VBOloadOneCube(verts, normals, myuvs, indexesAll);

	orbitX = 0; orbitY = 0; orbitZ = 0;

	glPushMatrix();

	glLoadIdentity();
	myglTranslate(transX, transY, transZ);
	myglRotate(rotateX, f2vt(1.0f), 0, 0);
	myglRotate(rotateY, 0, f2vt(1.0f), 0);
	myglRotate(rotateZ, 0, 0, f2vt(1.0f));

	myglGetv(GL_MODELVIEW_MATRIX, transform);

	glPopMatrix();

}

void Cubette::setTextureCoords(CUBECOLOR front, CUBECOLOR left, CUBECOLOR top, VERTTYPE *mytexcoords) {

	// everything starts out charcoal
	for (uint i=0; i<sizeof(uvs)/sizeof(uvs[0]); i+=2) {
		mytexcoords[i] = uvs[i];
		mytexcoords[i+1] = uvs[i+1] == 0 ? 6.0f/8.0f : 7.0f/8.0f;
	}

	// front
	for (uint i=0; i<sizeof(indexesFront)/sizeof(indexesFront[0]); i++) {
		int idx = indexesFront[i] * 2;
		mytexcoords[idx] = uvs[idx];
		mytexcoords[idx+1] = uvs[idx+1] == 0 ? (float)front/8.0f : (float)(front + 1)/8.0f;
	}

	// left
	for (uint i=0; i<sizeof(indexesLeft)/sizeof(indexesLeft[0]); i++) {
		int idx = indexesLeft[i] * 2;
		mytexcoords[idx] = uvs[idx];
		mytexcoords[idx+1] = uvs[idx+1] == 0 ? (float)left/8.0f : (float)(left + 1)/8.0f;
	}

	// top
	for (uint i=0; i<sizeof(indexesTop)/sizeof(indexesTop[0]); i++) {
		int idx = indexesTop[i] * 2;
		mytexcoords[idx] = uvs[idx];
		mytexcoords[idx+1] = uvs[idx+1] == 0 ? (float)top/8.0f : (float)(top + 1)/8.0f;
	}
}

void Cubette::Draw(FACE face) {

	glPushMatrix();

	switch (face) {

	case FRONT:
	case BACK:
		myglRotate(orbitZ, 0, 0, f2vt(1.0f));
		break;

	case LEFT:;
	case RIGHT:
		myglRotate(orbitX, f2vt(1.0f), 0, 0);
		break;

	case TOP:
	case BOTTOM:
		myglRotate(orbitY, 0, f2vt(1.0f), 0);
		break;

	case NOFACE: break;

	}

	myglMultMatrix(transform);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVBO);
	glDrawElements(GL_TRIANGLES, sizeof(indexesAll)/sizeof(indexesAll[0]), GL_UNSIGNED_SHORT, 0);

	glPopMatrix();
}

void Cubette::UpdateTransform() {

	glPushMatrix();

	glLoadIdentity();
	myglRotate(orbitX, f2vt(1.0f), 0, 0);
	myglRotate(orbitY, 0, f2vt(1.0f), 0);
	myglRotate(orbitZ, 0, 0, f2vt(1.0f));
	myglMultMatrix(transform);
	myglGetv(GL_MODELVIEW_MATRIX, transform);
	glPopMatrix();

	orbitX=0; orbitY=0; orbitZ=0;
}

Cubette::~Cubette() {
	// TODO Auto-generated destructor stub
}
