/*
 * Faces.c
 *
 *  Created on: Dec 25, 2009
 *      Author: enash
 */

#include "common.h"

int faceFront[9] = {0, 1, 2, 3, 4, 5, 6, 7, 8};
int faceBack[9] = {20, 19, 18, 23, 22, 21, 26, 25, 24};
int faceLeft[9] = {18, 9, 0, 21, 12, 3, 24, 15, 6};
int faceRight[9] = {2, 11, 20, 5, 14, 23, 8, 17, 26};
int faceTop[9] = {18, 19, 20, 9, 10, 11, 0, 1, 2};
int faceBottom[9] = { 6, 7, 8, 15, 16, 17, 24, 25, 26};

void mapFaces(FACE face, DIRECTION direction) {
	// this is after a "move"

	// remaps the faces after a 90 clockwise
	Cubette *temp;

	int iterations = (direction == CLOCKWISE) ? 1 : 3;

	for(int i=0; i<iterations; i++) {

		switch (face) {

		case FRONT: // 0>2, 1>5, 2>8, 3>1, 4>4, 5>7, 6>0, 7>3, 8>6
			temp = littleCubes[0]; littleCubes[0] = littleCubes[6]; littleCubes[6] = littleCubes[8]; littleCubes[8] = littleCubes[2]; littleCubes[2] = temp;
			temp = littleCubes[1]; littleCubes[1] = littleCubes[3]; littleCubes[3] = littleCubes[7]; littleCubes[7] = littleCubes[5]; littleCubes[5] = temp;
			break;

		case LEFT: // 18>0, 9>3, 0>6, 21>9, 12>12, 3>15, 24>18, 15>21, 6>24
			temp = littleCubes[0]; littleCubes[0] = littleCubes[18]; littleCubes[18] = littleCubes[24]; littleCubes[24] = littleCubes[6]; littleCubes[6] = temp;
			temp = littleCubes[3]; littleCubes[3] = littleCubes[9]; littleCubes[9] = littleCubes[21]; littleCubes[21] = littleCubes[15]; littleCubes[15] = temp;
			break;

		case TOP: // 18>20, 19>11, 20>2, 9>19, 10>10, 11>1, 0>18, 1>9, 2>0
			temp = littleCubes[20]; littleCubes[20] = littleCubes[18]; littleCubes[18] = littleCubes[0]; littleCubes[0] = littleCubes[2]; littleCubes[2] = temp;
			temp = littleCubes[11]; littleCubes[11] = littleCubes[19]; littleCubes[19] = littleCubes[9]; littleCubes[9] = littleCubes[1]; littleCubes[1] = temp;
			break;

		case BACK: // 20>18, 19>21, 18>24, 23>19, 22>22, 21>25, 26>20, 25>23, 24>26
			temp = littleCubes[18]; littleCubes[18] = littleCubes[20]; littleCubes[20] = littleCubes[26]; littleCubes[26] = littleCubes[24]; littleCubes[24] = temp;
			temp = littleCubes[21]; littleCubes[21] = littleCubes[19]; littleCubes[19] = littleCubes[23]; littleCubes[23] = littleCubes[25]; littleCubes[25] = temp;
			break;

		case RIGHT: // 2>20, 11>23, 20>26, 5>11, 14>14, 23>17, 8>2, 17>5, 26>8
			temp = littleCubes[20]; littleCubes[20] = littleCubes[2]; littleCubes[2] = littleCubes[8]; littleCubes[8] = littleCubes[26]; littleCubes[26] = temp;
			temp = littleCubes[23]; littleCubes[23] = littleCubes[11]; littleCubes[11] = littleCubes[5]; littleCubes[5] = littleCubes[17]; littleCubes[17] = temp;
			break;

		case BOTTOM: // 6>8, 7>17, 8>26, 15>7, 16>16, 17>25, 24>6, 25>15, 26>24
			temp = littleCubes[8]; littleCubes[8] = littleCubes[6]; littleCubes[6] = littleCubes[24]; littleCubes[24] = littleCubes[26]; littleCubes[26] = temp;
			temp = littleCubes[17]; littleCubes[17] = littleCubes[7]; littleCubes[7] = littleCubes[15]; littleCubes[15] = littleCubes[25]; littleCubes[25] = temp;
			break;

		case NOFACE:
			break;
		}
	}
}

void rotateFace(FACE face, VERTTYPE angle) {
	// this is for drawing

	int *faceCubes;
	char axis;
	float angleDir;

	switch (face) {

	case FRONT: faceCubes = faceFront; axis = 'z'; angleDir = -1; break;
	case BACK: faceCubes = faceBack; axis = 'z'; angleDir = 1; break;

	case LEFT: faceCubes = faceLeft; axis = 'x'; angleDir = 1; break;
	case RIGHT: faceCubes = faceRight; axis = 'x'; angleDir = -1; break;

	case TOP: faceCubes = faceTop; axis = 'y'; angleDir = -1; break;
	case BOTTOM: faceCubes = faceBottom; axis = 'y'; angleDir = 1; break;

	case NOFACE: break;

	}

	angle *= angleDir;

	for(int i=0; i<9; i++) {
		switch (axis) {
		case 'x':
			if(littleCubes[faceCubes[i]]->orbitX + angle >= 360.0f)
				littleCubes[faceCubes[i]]->orbitX += angle - 360.0f;
			else if(littleCubes[faceCubes[i]]->orbitX + angle < 0)
				littleCubes[faceCubes[i]]->orbitX += angle + 360.0f;
			else
				littleCubes[faceCubes[i]]->orbitX += angle;
			break;
		case 'y':
			if(littleCubes[faceCubes[i]]->orbitY + angle >= 360.0f)
				littleCubes[faceCubes[i]]->orbitY += angle - 360.0f;
			else if(littleCubes[faceCubes[i]]->orbitY + angle < 0)
				littleCubes[faceCubes[i]]->orbitY += angle + 360.0f;
			else
				littleCubes[faceCubes[i]]->orbitY += angle;
			break;
		case 'z':
			if(littleCubes[faceCubes[i]]->orbitZ + angle >= 360.0f)
				littleCubes[faceCubes[i]]->orbitZ += angle - 360.0f;
			else if(littleCubes[faceCubes[i]]->orbitZ + angle < 0)
				littleCubes[faceCubes[i]]->orbitZ += angle + 360.0f;
			else
				littleCubes[faceCubes[i]]->orbitZ += angle;
			break;
		}
	}

}
void updateFaceTransforms(FACE face) {
	// this is for drawing

	int *faceCubes;

	switch (face) {

	case FRONT: faceCubes = faceFront; break;
	case BACK: faceCubes = faceBack; break;

	case LEFT: faceCubes = faceLeft; break;
	case RIGHT: faceCubes = faceRight; break;

	case TOP: faceCubes = faceTop; break;
	case BOTTOM: faceCubes = faceBottom; break;

	case NOFACE: break;

	}

	for(int i=0; i<9; i++)
		littleCubes[faceCubes[i]]->UpdateTransform();
}
