/*
 * Shadow.cpp
 *
 *  Created on: Dec 26, 2009
 *      Author: enash
 */

#include "common.h"

extern VERTTYPE verts[];
extern GLushort indexesFront[];
extern GLushort indexesBack[];
extern GLushort indexesTop[];
extern GLushort indexesBottom[];
extern GLushort indexesLeft[];
extern GLushort indexesRight[];

Shadow::Shadow(Surface *shadowSurface) {

	surface = shadowSurface;

	// this vbo only needs verts
	VERTTYPE shadowVBOData[72]; // 3/vertex * 4 vertex/side * 6 sides

	for(uint i=0; i<sizeof(shadowVBOData)/sizeof(shadowVBOData[0]); i++)
		shadowVBOData[i] = verts[i] * 3;	// scale to 3 X 3

	glGenBuffers(1, &vboName);
	glBindBuffer(GL_ARRAY_BUFFER, vboName);
	glBufferData(GL_ARRAY_BUFFER, sizeof(shadowVBOData), shadowVBOData, GL_STATIC_DRAW);
}

void Shadow::Render() {

	surface->makeCurrent();

	// set environmental states
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);

	glDisableClientState(GL_NORMAL_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, vboName);
	glEnableClientState(GL_VERTEX_ARRAY);
	glVertexPointer(3, VERTTYPEENUM, 0, 0);

	//glEnable(GL_COLOR_MATERIAL);

	myglClearColor(f2vt(0.5f), f2vt(0.5f), f2vt(0.5f), f2vt(1.0f));
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//glColor4ub(255, 0, 0, 255);

	// RGB value is represntative of bit pattern of FACE number
	//enum FACE {FRONT, LEFT, TOP, BACK, RIGHT, BOTTOM, NOFACE};
	for(int i=0; i<6; i++) {

		switch((FACE)i) {
		case FRONT:
			glColor4ub(0, 0, 0, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesFront);
			break;
		case LEFT:
			glColor4ub(0, 0, 255, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesLeft);
			break;
		case TOP:
			glColor4ub(0, 255, 0, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesTop);
			break;
		case BACK:
			glColor4ub(0, 255, 255, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesBack);
			break;
		case RIGHT:
			glColor4ub(255, 0, 0, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesRight);
			break;
		case BOTTOM:
			glColor4ub(255, 0, 255, 255);
			glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, indexesBottom);
			break;
		case NOFACE: break;
		}
	}
	eglSwapBuffers(surface->eglDisplay, surface->eglSurface);
}

FACE Shadow::faceFromClick(int X, int Y) {

	surface->makeCurrent();

	GLubyte pixels[3];
	glReadPixels(X, Y, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixels);

	int face = 0;
	for(int i=0; i<3; i++) {
		face <<= 1;
		if(pixels[i] == 255)
			face |= 1;
		else if(pixels[i] != 0)
			return NOFACE;
	}
	return (FACE)face;
}

Shadow::~Shadow() {
	// TODO Auto-generated destructor stub
}
