/*
 * Surface.cpp
 *
 *  Created on: Dec 22, 2009
 *      Author: enash
 */
#include <stdio.h>
#include <stdlib.h>

#include "common.h"

EGLContext eglContext = 0;

Surface::Surface(NativeDisplayType display, NativeWindowType window, EGLint surface_type_bit, EGLint width, EGLint height) {

	/*
	 * display: X11 display
	 * window:	X11 window
	 * surface_type_bit: either EGL_WINDOW_BIT if the display window, or EGL_PBUFFER_BIT for the non-display pbuffer
	 * width:	if pbuffer, the width of the surface. ignored if window
	 * height:	if pbuffer, the height of the surface. ignored if window
	 */

	EGLint iMajorVersion, iMinorVersion;

	eglDisplay = eglGetDisplay(display);
	TestError("eglGetDisplay");

	if (!eglInitialize(eglDisplay, &iMajorVersion, &iMinorVersion)) {
		TestError("eglInitialize-1");
		printf("Error: eglInitialize() failed.\n");
		exit(1);
	}
	TestError("eglInitialize-2");

	// find a config thats a window (as compared with buffer)
	EGLConfig eglConfig;
	int iConfigs;
	EGLint configAttribs[] = {EGL_SURFACE_TYPE, surface_type_bit, EGL_DEPTH_SIZE, 16, EGL_NONE};

	if (!eglChooseConfig(eglDisplay, configAttribs, &eglConfig, 1, &iConfigs) || (iConfigs != 1)) {
		printf("Error: eglChooseConfig() failed.\n");
		exit(1);
	}

	if(eglContext == 0)
		eglContext = eglCreateContext(eglDisplay, eglConfig, NULL, NULL);
	TestError("eglCreateContext");

	if(surface_type_bit == EGL_WINDOW_BIT) {
		eglSurface = eglCreateWindowSurface(eglDisplay, eglConfig, window, NULL);
		TestError("eglCreateWindowSurface");
	}
	else if (surface_type_bit == EGL_PBUFFER_BIT) {
		EGLint pbufAttribs[] = {EGL_WIDTH, width, EGL_HEIGHT, height, EGL_NONE};
		eglSurface = eglCreatePbufferSurface(eglDisplay, eglConfig, pbufAttribs);
		TestError("eglCreatePbufferSurface");
	}
	else {
		printf("surface_type_bit is neither EGL_WINDOW_BIT or EGL_PBUFFER_BIT\n");
		exit(1);
	}

}

void Surface::makeCurrent() {

	eglMakeCurrent(eglDisplay, eglSurface, eglSurface, eglContext);
	TestError("xx");
}

Surface::~Surface() {
	// TODO Auto-generated constructor stub

}

