/*
 * EdsCube.cpp
 *
 *  Created on: Nov 27, 2009
 *      Author: enash
 */
#include "common.h"

void initLittleCubes();

int main(int argc, char **argv) {

	// init display, window, surfaces
	Xstuff *xstuff = new Xstuff(argc, argv);
	Surface *winSurface = new Surface((NativeDisplayType)xstuff->display, (NativeWindowType)xstuff->window, EGL_WINDOW_BIT, 0, 0);

	// pbuffer same size as window
	EGLint height, width;
	eglQuerySurface(winSurface->eglDisplay, winSurface->eglSurface, EGL_WIDTH, &width);
	eglQuerySurface(winSurface->eglDisplay, winSurface->eglSurface, EGL_HEIGHT, &height);
	Surface *pbufSurface = new Surface((NativeDisplayType)xstuff->display, (NativeWindowType)xstuff->window, EGL_PBUFFER_BIT, width, height);

	winSurface->makeCurrent();

	// lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	VERTTYPE lightAmbient[] = { f2vt(0.5f), f2vt(0.5f), f2vt(0.5f), f2vt(1.0f) };
	myglLightv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	VERTTYPE lightDiffuse[] = { f2vt(0.75f), f2vt(0.75f), f2vt(0.75f), f2vt(1.0f) };
	myglLightv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	//VERTTYPE lightDirection[] = { f2vt(-2.0f), f2vt(2.0f), f2vt(1.0), 0 }; // w=0 means directional
	//myglLightv(GL_LIGHT0, GL_POSITION, lightDirection);

	// for 3d rendering
	glEnable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	GLint parms[4]; //x,y,width,height
	glGetIntegerv(GL_VIEWPORT, parms);
	float aspect = (float)parms[2] / (float)parms[3];
	myglFrustum(f2vt(-1.0), f2vt(1.0), f2vt(-1.0/aspect), f2vt(1.0/aspect), 5.0, 50.0);

	glMatrixMode(GL_MODELVIEW);

	Cube *cube = new Cube(winSurface, 15.0);
	Shadow *shadow = new Shadow(pbufSurface);

	XEvent xevent;
	int x, y, lastX, lastY;
	float rotateX = 45;
	float rotateY = 45;
	FACE face;
	char keys[10];

	while(1) {

		glLoadIdentity();
		myglTranslate(f2vt(0.0f), f2vt(0.0f), f2vt(-20.0f)); // move into view
		myglRotate(rotateX, f2vt(1.0f), f2vt(0.0f), f2vt(0.0f));
		myglRotate(rotateY, f2vt(0.0f), f2vt(1.0f), f2vt(0.0f));

		cube->Render(NOFACE, 0);
		shadow->Render();

		XNextEvent(xstuff->display, &xevent);
		switch(xevent.type) {

		case KeyPress:
			XLookupString(&xevent.xkey, keys, sizeof(keys), 0, 0);
			printf("Keycode: %d Keys: %s\n", xevent.xkey.keycode, keys);
			//if(keys[0] == 'q')
				//exit(0);
			break;

		case ButtonPress:
			lastY = y = height - xevent.xbutton.y; // adjust for top to bottom
			lastX = x = xevent.xbutton.x; // adjust for top to bottom
			face = shadow->faceFromClick(x, y);
			//face = NOFACE;
			if(face != NOFACE) {
				cube->Render(face, 1.0);
				mapFaces(face, CLOCKWISE);
			}
			break;

		case MotionNotify:
			if(face == NOFACE) {
				y = height - xevent.xmotion.y; // adjust for top to bottom
				x = xevent.xmotion.x; // adjust for top to bottom

				rotateY += ((float)(x - lastX)/(float)width)*360.0f;
				rotateX += ((float)(y - lastY)/(float)height)*360.0f;

				rotateX = rotateX > 360 ? rotateX - 360 : (rotateX  < 0 ? rotateX + 360 : rotateX);
				rotateY = rotateY > 360 ? rotateY - 360 : (rotateY  < 0 ? rotateY + 360 : rotateY);

				lastX = x;
				lastY = y;
			}
			break;
		}
	}
}

void TestError(char* pszLocation) {

	bool ok = true;
	char strErr[50];

	// GL error
	GLenum glErr = glGetError();
	switch (glErr) {
	case GL_NO_ERROR: break;
	case GL_INVALID_ENUM: strcpy(strErr, "GL_INVALID_ENUM"); break;
	case GL_INVALID_VALUE: strcpy(strErr, "GL_INVALID_VALUE"); break;
	case GL_INVALID_OPERATION: strcpy(strErr, "GL_INVALID_OPERATION"); break;
	case GL_STACK_OVERFLOW: strcpy(strErr, "GL_STACK_OVERFLOW"); break;
	case GL_OUT_OF_MEMORY: strcpy(strErr, "GL_OUT_OF_MEMORY"); break;
	}
	if (glErr != GL_NO_ERROR) {
		printf("glGetError: %s failed (%d) %s\n", pszLocation, glErr, strErr);
		ok = false;
	}

	// EGL error
	EGLint eglErr = eglGetError();
	switch (eglErr) {
	case EGL_SUCCESS: break;
	case EGL_NOT_INITIALIZED: strcpy(strErr, "EGL_NOT_INITIALIZED"); break;
	case EGL_BAD_ACCESS: strcpy(strErr, "EGL_BAD_ACCESS"); break;
	case EGL_BAD_ALLOC: strcpy(strErr, "EGL_BAD_ALLOC"); break;
	case EGL_BAD_ATTRIBUTE: strcpy(strErr, "EGL_BAD_ATTRIBUTE"); break;
	case EGL_BAD_CONTEXT: strcpy(strErr, "EGL_BAD_CONTEXT"); break;
	case EGL_BAD_CONFIG: strcpy(strErr, "EGL_BAD_CONFIG"); break;
	case EGL_BAD_CURRENT_SURFACE: strcpy(strErr, "EGL_BAD_CURRENT_SURFACE"); break;
	case EGL_BAD_DISPLAY: strcpy(strErr, "EGL_BAD_DISPLAY"); break;
	case EGL_BAD_SURFACE: strcpy(strErr, "EGL_BAD_SURFACE"); break;
	case EGL_BAD_MATCH: strcpy(strErr, "EGL_BAD_MATCH"); break;
	case EGL_BAD_PARAMETER: strcpy(strErr, "EGL_BAD_PARAMETER"); break;
	case EGL_BAD_NATIVE_PIXMAP: strcpy(strErr, "EGL_BAD_NATIVE_PIXMAP"); break;
	case EGL_BAD_NATIVE_WINDOW: strcpy(strErr, "EGL_BAD_NATIVE_WINDOW"); break;
	case EGL_CONTEXT_LOST: strcpy(strErr, "EGL_CONTEXT_LOST"); break;
	}
	if (eglErr != EGL_SUCCESS) {
		printf("eglGetError: %s failed (%d) %s\n", pszLocation, eglErr, strErr);
		ok = false;
	}

	if(!ok)
		exit(1);
}





